#!/usr/bin/python
#######################################################################
# File  : Install.py
# Author: Souheil CHELFOUH
#         mailto:souheil@chelfouh.com
#         http://www.trollskog.com
# Date  : 2006-02-06
#######################################################################

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'

from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes
from Products.CMFDynamicViewFTI.migrate import migrateFTIs

from Products.TPArtist.config import *
from Products.TPArtist.Extensions.SetupTools import *

# My configlets 
configlets = (
    {'id'           : PROJECTNAME,
     'name'         : 'Artists management',
     'action'       : 'string:${portal_url}/prefs_TPArtist_edit_form',
     'condition'    : '',
     'category'     : 'Products',
     'visible'      : 1,
     'appId'        : PROJECTNAME,
     'permission'   : 'Manage portal',
     'imageUrl'     : 'artist_tool.gif' },
    )


def install(self):
    """
    Install TPArtist: install and configure the module 'TPArtist'
    """
    out = StringIO()

    print >> out, "Installation - TPArtist module"

    # Install types
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    print >> out, "Installed types"

    # Install tools
    setupTool(self, "portal_artist", PROJECTNAME)
    setupConfiglet(self, configlets)

    # Install skin
    install_subskin(self, out, product_globals)
    print >> out, "Installed skin"

    # Migrate FTI, to make sure we get the necessary infrastructure for the
    # 'display' menu to work.
    migrated = migrateFTIs(self, product=PROJECTNAME)
    print >>out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    # Enable portal_factory
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    trollTypes = ['Artist','Album','Track', 'Review']
    for trollType in trollTypes:
        if trollType not in types:
            types.append(trollType)
            factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added TPArtist types to portal_factory"


    # Hide types to navigation
    hideTypes = ['Review',]
    hideMe(self, hideTypes, out)

    print >> out, "Hidded TPArtist types : %s" % ','.join(hideTypes)

    # Add some basic keywords
    portal_artist = getToolByName(self, 'portal_artist')
    keys = ('Configure in Site Setup / Artist Management (you can remove this line)',)
    if not portal_artist.getValues('medias'):
        portal_artist.setType(keys, True, 'Album')
    if not portal_artist.getValues('styles'):
        portal_artist.setType(keys, True, 'Artist')

    
    return out.getvalue()



def uninstall(self):
    out = StringIO()

    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            configTool.unregisterConfiglet(conf['id'])
            out.write('Removed configlet %s\n' % conf['id'])

    return out.getvalue()
