#!/usr/bin/python
# -*- coding: utf-8 -*-
#######################################################################
# File  : album.py
# Author: Souheil CHELFOUH
#         mailto:souheil@chelfouh.com
#         http://www.trollskog.com
# Date  : 2006-02-11
#######################################################################

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'


import string
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.content.folder import ATFolder
from Products.TPArtist.schemas import AlbumSchema
from Products.TPArtist.interfaces import IAlbum


class Album(ATFolder):
    """A music album"""

    # Configuration of my type
    portal_type = metal_type    = "Album"
    archetype_name              = "Album"
    content_icon                = "album.gif"
    schema                      = AlbumSchema
    typeDescription             = "A music album"
    typeDescMsgId               = "at_label_album"
    global_allow                = False
    allow_discussion            = True
    
    # Contents
    allowed_content_types       = ['Track','Review',]

    # Views
    default_view                = 'album_view'
    immediate_view              = 'album_view'
    suppl_views                 = ()

    # Aditional properties
    _at_rename_after_creation   = True    # Get me a nice ID !

    # Still mystical for me
    __implements__ = OrderedBaseFolder.__implements__ + \
                     ATFolder.__implements__ + \
                     (IAlbum,)

    # Get the standard actions (tabs)
    actions = ATFolder.actions
    
    # Security
    security = ClassSecurityInfo()


    security.declareProtected('View', 'canSetDefaultPage')
    def canSetDefaultPage(self):
        """
        No Default Page can be set for this folderish type
        """
        return False


    security.declareProtected('View', 'getSubjectVocabulary')
    def getSubjectVocabulary(self):
        """
        Queries the TPArtist tool for the Subject vocabulary
        """
        return getToolByName(self, 'portal_artist').getValues(type = 'medias')


    security.declareProtected('View', 'getAlbumTitle')
    def getAlbumTitle(self):
        """
        Returns the album's title
        """
        return self.Title()


    ###################
    ## Handle Tracks ##
    ###################

    security.declareProtected('View', 'getAlbumTracks')
    def getAlbumTracks(self):
        """
        Returns the list of the albums' tracks
        """
        contentFilter = { "portal_type" : ("Track",) }
        return self.getFolderContents(contentFilter)


    security.declareProtected('View', 'getNbTracks')
    def getNbTracks(self):
        """
        Returns the number of tracks in the album
        """
        return len(self.getAlbumTracks())


    security.declareProtected('View', 'getGlobalRunningTime')
    def getGlobalRunningTime(self):
        """
        Get the running time
        """
        ### Fix me... Rising all the elements is not so performance friendly ###
        
        tracks  = self.getAlbumTracks()
        seconds = int(0)

        if not tracks:
            return None
        
        for track in tracks:
            myTrack  = track.getObject()
            duration = myTrack.getDuration()
            if duration:
                myHour, myMin, mySec = duration.split(':')
                seconds += int(mySec) + \
                           int(myMin) * 60 + \
                           int(myHour) * 3600

        return "%02d:%02d:%02d" % (seconds / 3600,
                                   (seconds % 3600) / 60,
                                   seconds % 60)
    

    ####################
    ## Handle Reviews ##
    ####################

    security.declareProtected('View', 'getLevel')
    def getLevel(self):
        """
        return object depth - Album is top level
        """
        return 0


    security.declareProtected('View', 'getReviewedItem')
    def getReviewedItem(self):
        """
        Return this album
        Available for the reviews
        """
        return self

    security.declareProtected('View', 'getAllPosts')
    def getAllPosts(self):
        """
        Sends back all the posts of this album
        """
        cat = getToolByName(self, 'portal_catalog')
        post_path = string.join(self.getPhysicalPath(), '/')
        return cat(path = post_path, portal_type = ('Review',),)

   
    security.declareProtected('View', 'getNumberOfPostsInside')
    def getNumberOfPostsInside(self):
        """
        Get the number of posts/replies within the post.
        For displaying purpose.
        """   
        return len(self.getAllPosts())


    security.declareProtected('View', 'getNumberOfPostsInside')
    def getGlobalMark(self):
        """
        Get the average mark from all the posts
        """
        ### Fix me... Rising all the elements is not so performance friendly ###
        
        posts   = self.getAllPosts()
        nbposts = len(posts)
        marks   = float(0)

        if not nbposts:
            return False
        
        for post in posts:
            myPost = post.getObject()
            marks += float(myPost.getMark())
        
        return float(marks / nbposts)
    

registerType(Album)
