#!/usr/bin/python
# -*- coding: utf-8 -*-
#######################################################################
# File  : artist.py
# Author: Souheil CHELFOUH
#         mailto:souheil@chelfouh.com
#         http://www.trollskog.com
# Date  : 2006-02-11
#######################################################################

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'


from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.content.document import ATDocument
from Products.TPArtist.schemas import ArtistSchema
from Products.TPArtist.interfaces import IArtist


class Artist(OrderedBaseFolder, ATDocument):
    """A musical artist / band"""
    
    # Configuration of my type
    portal_type = metal_type    = "Artist"
    archetype_name              = "Artist"
    content_icon                = "artist.gif"
    schema                      = ArtistSchema
    typeDescription             = "Musical artist / band"
    typeDescMsgId               = "at_label_artist"
    
    # Contents
    allowed_content_types       = ['AlbumPhoto','Album',]

    # Views
    default_view                = 'artist_view'
    immediate_view              = 'artist_view'
    suppl_views                 = ()

    # Additional properties
    _at_rename_after_creation   = True    # Get me a nice ID !

    # Still mystical for me
    __implements__ = OrderedBaseFolder.__implements__ + \
                     ATDocument.__implements__ + \
                     (IArtist,)

    # Default actions
    custom  = ({'id':'cddb',
                'name': 'CDDB Feed',
                'action':'artist_cddb',
                'permissions': (CMFCorePermissions.ModifyPortalContent,)
                },)
    actions = ATDocument.actions
    actions = actions + custom
            
    # Security
    security = ClassSecurityInfo()

    ####################################
    # CDDB persistant cache
    # variable : cddb
    ####################################
    
    def isCddbCached(self):
        """
        Test if there's already a cached request
        """
        if hasattr(self, 'cddb'):
            return True
        return False

    def getCddbCache(self):
        """
        Return the content of the cached CDDB request
        """
        if hasattr(self, 'cddb'):
            return self.cddb
        return None

    def setCddbCache(self, toCache):
        """
        Cache the request in a non persistant properties
        """
        if toCache:
            self.cddb = toCache
            return True
        return False

    def getStyleByCdid(self, cdid):
        """
        Get the style of the album referenced by 'cdid'
        in the cache of the item CDDB rquest
        """
        if hasattr(self, 'cddb'):
            cache = self.cddb
            for cd in cache:
                if cd[2] == cdid:
                    return cd[1]
        return None


    ####################################

    def getArtistName(self):
        """
        Return the name of the artist
        """
        return self.Title()


    
    def getWebsite(self):
        """
        Returns the formatted artist's website url
        """
        url = self.getUrl()
        #if (url.count('http://') == 0):
        #    return 'http://' + url
        return url


    security.declareProtected('View', 'getArtistAlbums')
    def getArtistAlbums(self):
        """
        Returns the list of the artist's albums
        """
        contentFilter = { "portal_type" : ("Album",) }
        return self.getFolderContents(contentFilter)
        

    def getNbAlbums(self):
        """
        Returns the number of albums of the artist
        """
        return len(self.getArtistAlbums())

    def getSubjectVocabulary(self):
        """
        Queries the TPArtist tool for the Subject vocabulary
        """
        return getToolByName(self, 'portal_artist').getValues(type = 'styles')
    
    def canSetDefaultPage(self):
        """
        No Default Page can be set for this folderish type
        """
        return False
    
registerType(Artist)



