#!/usr/bin/python
# -*- coding: utf-8 -*-
#######################################################################
# File  : duration_field.py
# Author: Souheil CHELFOUH
#         mailto:souheil@chelfouh.com
#         http://www.trollskog.com
# Date  : 2006-03-13
#######################################################################

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'

import re
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringField, StringWidget, ObjectField
from Products.validation.interfaces import ivalidator
from Products.Archetypes.Registry import registerField, registerWidget, registerPropertyType


class DurationWidget(StringWidget):
    """
    """
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro_edit' : "duration_widget",
        })

    security  = ClassSecurityInfo()
    
    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker="00",
                     emptyReturnsMarker=False):
        """form processing for my duration"""

        hours   = form.get('%s_hrs' % field.getName(), empty_marker)
        minutes = form.get('%s_min' % field.getName(), empty_marker)
        seconds = form.get('%s_sec' % field.getName(), empty_marker)

        value   = "%s:%s:%s" % (hours, minutes, seconds)

        return value, {}


class DurationField(ObjectField):
    """A simple duration field"""
    
    security    = ClassSecurityInfo()
    
    __implements__ = ObjectField.__implements__
    
    _properties = StringField._properties.copy()
    _properties.update({
        'type' : 'duration',
        'widget' : DurationWidget,
        'default': '00:00:00',
        'validators' : ('isValidDuration',),
        })

    def set(self, instance, value, **kwargs):
        """
        """
        hours, minutes, seconds = value.split(':')
        value = "%02d:%02d:%02d" % (int(hours), int(minutes), int(seconds))
        ObjectField.set(self, instance, value, **kwargs)


class DurationValidator:
    """
    Please, validate my duration
    """
    
    __implements__ = (ivalidator,)
    
    def __init__(self, name):
        """
        """
        self.name = name


    def __call__(self, value, *args, **kwargs):
        """
        """
        hours, minutes, seconds = value.split(':')
        pattern = re.compile(r'^[0-9]{0,2}$')
       
        h = pattern.match(hours)
        m = pattern.match(minutes)
        s = pattern.match(seconds)

        # They are not integers
        if not (h and m and s):
            return ("Validation failed: incorrect duration")

        # They are not in 
        if int(minutes) not in range(60) or int(seconds) not in range(60):
            return ("Validation failed: incorrect duration")
        
        return True


registerWidget(DurationWidget,
               title='Duration',
               description="Renders a minutes + seconds field",
               used_for=('Products.TPArtist.fields.DurationField',)
               )

registerField(DurationField,
              title='Duration',
              description='Used for storing durations.')
