#!/usr/bin/python
# -*- coding: utf-8 -*-
#######################################################################
# File  : url_field.py
# Author: Souheil CHELFOUH
#         mailto:souheil@chelfouh.com
#         http://www.trollskog.com
# Date  : 2006-03-13
#######################################################################

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'


import re
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringField, StringWidget
from Products.validation.interfaces import ivalidator
from Products.Archetypes.Registry import registerField, registerWidget, registerPropertyType


class UrlWidget(StringWidget):
    """
    """
    _properties = StringWidget._properties.copy()
    _properties.update({
        'target' : '',
        })


class UrlField(StringField):
    """A very simple URL field"""
    
    _properties = StringField._properties.copy()
    _properties.update({
        'type' : 'url',
        'widget' : UrlWidget,
        'validators' : ('isPartialUrl',),
        'default_protocol' : 'http',
        })

    # Secure my methods !
    security = ClassSecurityInfo()

    security.declarePrivate('fixProtocol')
    def fixProtocol(self, value):
        if not value or '://' in value:
            return value
        return '://'.join([self.default_protocol, value])


    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        value = self.fixProtocol(value)
        StringField.set(self, instance, value, **kwargs)


class PartialUrlValidator:
    """
    Please, validate my URL
    """
    
    __implements__ = (ivalidator,)
    
    def __init__(self, name):
        """
        """
        self.name = name


    def __call__(self, value, *args, **kwargs):
        """
        """
        field = kwargs.get('field')
        if hasattr(field, 'fixProtocol'):
            value = field.fixProtocol(value)

        pattern = re.compile(r'(ht|f)tps?://[^\s\r\n]+')
        m = pattern.match(value)
        if not m:
            return ("Validation failed: incorrect url." % (self.name, repr(value)))
        
        return True


registerWidget(UrlWidget,
               title='Url',
               description="Renders a Url in an anchor tag.",
               used_for=('Products.TPArtist.fields.UrlField',)
               )

registerField(UrlField,
              title='Url',
              description='Used for storing urls.')


registerPropertyType('target', 'string', UrlWidget)
registerPropertyType('default_protocol', 'string', UrlField)
